@@ -391,12 +391,14 @@ struct pw_core *pw_core_new(struct pw_loop *main_loop, struct pw_properties *pro
 	spa_graph_init(&this->rt.graph);
 	spa_graph_set_callbacks(&this->rt.graph, &spa_graph_impl_default, NULL);
 
+	this->dbus_iface = pw_get_spa_dbus(this->main_loop);
+
 	this->support[0] = SPA_SUPPORT_INIT(SPA_TYPE__TypeMap, this->type.map);
 	this->support[1] = SPA_SUPPORT_INIT(SPA_TYPE_LOOP__DataLoop, this->data_loop->loop);
 	this->support[2] = SPA_SUPPORT_INIT(SPA_TYPE_LOOP__MainLoop, this->main_loop->loop);
 	this->support[3] = SPA_SUPPORT_INIT(SPA_TYPE__LoopUtils, this->main_loop->utils);
 	this->support[4] = SPA_SUPPORT_INIT(SPA_TYPE__Log, pw_log_get());
-	this->support[5] = SPA_SUPPORT_INIT(SPA_TYPE__DBus, pw_get_spa_dbus(this->main_loop));
+	this->support[5] = SPA_SUPPORT_INIT(SPA_TYPE__DBus, this->dbus_iface);
 	this->n_support = 6;
 
 	pw_log_debug("%p", this->support[5].data);
@@ -494,6 +496,8 @@ void pw_core_destroy(struct pw_core *core)
 
 	pw_data_loop_destroy(core->data_loop_impl);
 
+	pw_release_spa_dbus(core->dbus_iface);
+
 	pw_properties_free(core->properties);
 
 	pw_map_clear(&core->globals);
@@ -553,10 +557,15 @@ const struct pw_properties *pw_core_get_properties(struct pw_core *core)
 int pw_core_update_properties(struct pw_core *core, const struct spa_dict *dict)
 {
 	struct pw_resource *resource;
-	uint32_t i;
+	uint32_t i, changed = 0;
 
 	for (i = 0; i < dict->n_items; i++)
-		pw_properties_set(core->properties, dict->items[i].key, dict->items[i].value);
+		changed += pw_properties_set(core->properties, dict->items[i].key, dict->items[i].value);
+
+	pw_log_debug("core %p: updated %d properties", core, changed);
+
+	if (!changed)
+		return 0;
 
 	core->info.change_mask = PW_CORE_CHANGE_MASK_PROPS;
 	core->info.props = &core->properties->dict;
@@ -568,7 +577,7 @@ int pw_core_update_properties(struct pw_core *core, const struct spa_dict *dict)
 
 	core->info.change_mask = 0;
 
-	return 0;
+	return changed;
 }
 
 int pw_core_for_each_global(struct pw_core *core,
